// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Shape class

package geometry;

import java.io.*;
import java.awt.*;
import vrml.external.*;
import vrml.external.field.*;
import vrml.external.exception.*;
import geometry.VrmlObject;

public class Material extends VrmlObject
{
	EventInSFColor	_set_diffuse;
	EventInSFColor	_set_emissive;
	EventInSFColor	_set_specular;
	EventInSFFloat	_set_ambient;
	EventInSFFloat	_set_shine;
	EventInSFFloat	_set_transparent;
	float[]	diffuse;
	float[]	emissive;
	float[]	specular;
	float	ambient = 0.2f;
	float	shininess = 0.2f;
	float	transparency = 0;
	boolean diffuse_changed = false;
	boolean emissive_changed = false;
	boolean specular_changed = false;

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Material()
	{
		// setup default values
		diffuse = new float[3];
		diffuse[0] = 0.8f;
		diffuse[1] = 0.8f;
		diffuse[2] = 0.8f;

		emissive = new float[3];
		emissive[0] = 0;
		emissive[1] = 0;
		emissive[2] = 0;

		specular = new float[3];
		specular[0] = 0;
		specular[1] = 0;
		specular[2] = 0;

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Material {}")[0];
		_set_ambient =
			(EventInSFFloat)node.getEventIn("ambientIntensity");
		_set_diffuse = (EventInSFColor)node.getEventIn("diffuseColor");
		_set_emissive = (EventInSFColor)node.getEventIn("emissiveColor");
		_set_specular = (EventInSFColor)node.getEventIn("specularColor");
		_set_shine = (EventInSFFloat)node.getEventIn("shininess");
		_set_transparent =
			(EventInSFFloat)node.getEventIn("transparency");

		have_browser = true;
	}

	public Material(Browser b)
	{
		// setup default values
		diffuse = new float[3];
		diffuse[0] = 0.8f;
		diffuse[1] = 0.8f;
		diffuse[2] = 0.8f;

		emissive = new float[3];
		emissive[0] = 0;
		emissive[1] = 0;
		emissive[2] = 0;

		specular = new float[3];
		specular[0] = 0;
		specular[1] = 0;
		specular[2] = 0;

		if(b == null)
			return;

		node = b.createVrmlFromString("Material {}")[0];
		_set_ambient =
			(EventInSFFloat)node.getEventIn("ambientIntensity");
		_set_diffuse = (EventInSFColor)node.getEventIn("diffuseColor");
		_set_emissive = (EventInSFColor)node.getEventIn("emissiveColor");
		_set_specular = (EventInSFColor)node.getEventIn("specularColor");
		_set_shine = (EventInSFFloat)node.getEventIn("shininess");
		_set_transparent =
			(EventInSFFloat)node.getEventIn("transparency");

		have_browser = true;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "material ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Material {");

		// print the contents of the shape node
		if(ambient != 0.2f)
			fp.println(buffer.toString() + " ambient " + ambient);
		if(shininess != 0.2f)
			fp.println(buffer.toString() +
				   " shininess " + shininess);
		if(transparency != 0)
			fp.println(buffer.toString() +
				   " transparency " + ambient);
		if(diffuse_changed)
			fp.println(buffer.toString() +
				   " diffuseColor " +
				   diffuse[0] + " " +
				   diffuse[1] + " " +
				   diffuse[2]);

		if(emissive_changed)
			fp.println(buffer.toString() +
				   " emissiveColor " +
				   emissive[0] + " " +
				   emissive[1] + " " +
				   emissive[2]);

		if(specular_changed)
			fp.println(buffer.toString() +
				   " specularColor " +
				   specular[0] + " " +
				   specular[1] + " " +
				   specular[2]);

		fp.println(buffer.toString() + "}");
	}

	public void set_ambientIntensity(float val)
	{
		ambient = val;
		if(have_browser)
			_set_ambient.setValue(val);
	}

	public void set_shininess(float val)
	{
		shininess = val;
		if(have_browser)
			_set_shine.setValue(val);
	}

	public void set_transparency(float val)
	{
		transparency = val;
		if(have_browser)
			_set_transparent.setValue(val);
	}

	public void set_diffuseColor(float red, float green, float blue)
	{
		diffuse[0] = red;
		diffuse[1] = green;
		diffuse[2] = blue;

		if(have_browser)
			_set_diffuse.setValue(diffuse);

		if((red == 0.8) && (green == 0.8) && (blue == 0.8))
			diffuse_changed = false;
		else
			diffuse_changed = true;
	}

	public void set_emissiveColor(float red, float green, float blue)
	{
		emissive[0] = red;
		emissive[1] = green;
		emissive[2] = blue;

		if(have_browser)
			_set_emissive.setValue(emissive);

		if((red == 0) && (green == 0) && (blue != 0))
			emissive_changed = false;
		else
			emissive_changed = true;
	}

	public void set_specularColor(float red, float green, float blue)
	{
		specular[0] = red;
		specular[1] = green;
		specular[2] = blue;

		if(have_browser)
			_set_specular.setValue(specular);

		if((red == 0) && (green == 0) && (blue == 0))
			specular_changed = false;
		else
			specular_changed = true;
	}
}
